/*=============================================================================
  Copyright (c) 1999-2003 pmG Worldwide, LLC.

  Author: Lyle Milton
  Date: 03/14/03

  Description: Simple effect that causes a point to lag behind the motion of the 
  object based on their distance from the pivot.
  See the tutorials section of the docs for a detailed description of this file.	

=============================================================================*/
#include "LagPoints.h"

//Effect & Interface Callbacks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
static FXvoid LagPointDataInit(LagPointState *state);
static FX_CONCALLBACK(Lag_WeightPopup);
static FX_CONCALLBACK(LagScan_Bool);
static FX_EFFECTSCAN(EffectScan, LagPointState, state);
static FX_DISPLACESCAN(DisplaceScan, LagPointState, state);

static FXcontrol	block, c1, c2, c3, c4;

//=======================================================================================
//LagPointAccess
//		effect access message callback
//=======================================================================================
FX_ACCESSFUNC(LagPointsAccess)
{
	FXeffect		effect = (FXeffect)ID;
	LagPointState	*state = NULL;

	switch( level )
	{

	case ACCESS_OBJECT:
		switch( entry )
		{
		case O_CREATE:
			{
				// Allocate memory for our state structure
				if( !(state = (LagPointState *)calloc(1, sizeof(LagPointState))) )
					return ENTRY_CONTINUE;

				// User defined state initialization function
				LagPointDataInit(state);

				// Attach the state data to the effect
				fxObjectSetTypeData( effect, state, FX_NOFLAG );

				// Set up channel
				fxChannelSetup( effect, "lag", FX_CHAN_0, FX_CHANTYPE_VALUE, state->lag, 0.0, 0.0, FX_NOFLAG);

				// Link channel 0 to state->scale
				fxChannelDataLink( effect, FX_CHAN_0, FX_ARG_DOUBLE, &state->lag, FX_NOFLAG);

			}// O_CREATE
			break;

		case O_DESTROY:
			{
				if( state = fxObjectGetTypeData( effect, FX_NOFLAG ) )
				{
					// Free memory
					free(state);

					// Set effect's state data as NULL
					fxObjectSetTypeData( effect, NULL, FX_NOFLAG);
				}
			}//O_DESTROY
			break;

		case O_SAVE:
			{
				FX_Format *f;

				if( state = fxObjectGetTypeData( effect, FX_NOFLAG ) )
				{
					// Create new format called "LagData"
					f = fxFormatNew(ai, "LagData", FX_NOFLAG);
						
						// Store state->weight in this format and call it "weight"
						fxArgSaveObj(f, "weight", state->weight_obj);

						// Store state->scan also, calling it "scan"
						fxArgSaveInt(f, "scan", state->scan);

				}
			}//O_SAVE
			break;

		case O_LOAD:
			{
				FX_Format *f;

				if( state = fxObjectGetTypeData(effect, FX_NOFLAG) )
				{
					// initialize data prior to loading
					LagPointDataInit(state); 
					
					// Retrieve our format from ai
					f = fxFormatGet( ai, "LagData" );

						// Retrieve state->weight_obj from the format by name (weight)
						fxArgLoadObj(f, "weight", &state->weight_obj);

						// Retrieve state->scan from the format by name (scan)
						fxArgLoadInt(f, "scan", &state->scan);

				}
			}//O_LOAD
			break;

		}//ACCESS_OBJECT
		break;

	case ACCESS_PROCESS:
		switch(entry)
		{
		case P_POST_POINT_DISPLACE:
			if( state = fxObjectGetTypeData( effect, FX_NOFLAG ) )
			{
				state->frame = NOW;

				// Process each target, EffectScan() is called once for each attached target
				fxEffectTargetScan( effect, &EffectScan, state, FX_NOFLAG );
			}
			break;

		}//ACCESS_PROECESS
		break;

	case ACCESS_INTERFACE:
		switch(entry)
		{
			case IN_CREATE:
				{	
					block = fxCC_Block("LagPoints", NULL, ACC_NONE, FX_NOFLAG);

						c1 = fxCC_ChannelFloat("Lag", NULL, ACC_NONE, FX_CHAN_0, 0); 
							 fxConSetPos(c1, 75, 4);
						c2 = fxCC_WeightPopup("Weight", &Lag_WeightPopup, ACC_VALUE|ACC_UPDATE, 175);
						c3 = fxCC_Bool("Use PointScan", &LagScan_Bool, ACC_VALUE|ACC_UPDATE);

					fxInterfaceBlockSet(FX_MODE_ANIMATE, NULL, NULL, block);
					fxInterfaceBlockSet(FX_MODE_ANIMATE, FX_MODE_ANIMATE, FX_MBLOCK_SPLINE, FX_NULLID);
					fxInterfaceBlockSet(FX_MODE_SETUP, NULL, NULL, block);

				}//IN_CREATE
				break;
		}//ACCESS_INTERFACE
		break;

	case ACCESS_SYSTEM:
		switch( entry )
		{
		case S_MODULE_INIT:
			{
				// This effect will be represented by a slider in messiah's world view
				fxInitEffectManipulator(FX_MANIPULATOR_SLIDER, FX_NOFLAG);

				// Give the effect's icon a color
				fxInitEffectColor((FXubyte)200, (FXubyte)220, (FXubyte)0);

				// Set the type of tool used by the effect, a null in this case
				fxInitEffectTool(FX_TOOLTYPE_NULL, FX_NOFLAG);
				
			}// S_MODULE_INIT
			break;
		}// ACCESS_SYSTEM
		break;

	}// switch level

	return ENTRY_OK;
}


//=======================================================================================
//LagPointDataInit
//		initialize effect data parameters
//=======================================================================================
FXvoid LagPointDataInit(LagPointState *state)
{
	state->lag = 0.0;
	state->piv_obj = FX_NULLID;
	state->use_piv = FX_FALSE;
	return;
}


//=======================================================================================
//Lag_WeightPopup
//		control callback to get/set weight object
//=======================================================================================
static FX_CONCALLBACK(Lag_WeightPopup)
// FXint control_func(FXcontrol ctl, FXarb ctl_data, FX_Arg *arg, FXint64 entry )
{
	FXeffect		effect = (FXeffect)ctl_data;
	LagPointState	*state = fxObjectGetTypeData(effect, FX_NOFLAG);

	if (!state) return ENTRY_CONTINUE;

	switch(entry)
	{
		case ACC_VALUE:
			fxConGetObj(ctl, &state->weight_obj);
			break;

		case ACC_UPDATE:
			fxConSetObj(ctl, state->weight_obj);
			break;

		default:
			break;
	}
	return ENTRY_CONTINUE;
}


//=======================================================================================
//LagScan_Bool
//		control callback to get/set point scan mode.
//=======================================================================================
static FX_CONCALLBACK(LagScan_Bool)
// FXint control_func(FXcontrol ctl, FXarb ctl_data, FX_Arg *arg, FXint64 entry )
{
	FXeffect		effect = (FXeffect)ctl_data;
	LagPointState	*state = fxObjectGetTypeData(effect, FX_NOFLAG);

	if (!state) return ENTRY_CONTINUE;

	switch(entry)
	{
		case ACC_VALUE:
			fxConGetInt(ctl, &state->scan);
			break;

		case ACC_UPDATE:
			fxConSetInt(ctl, state->scan);
			break;

		default:
			break;
	}
	return ENTRY_CONTINUE;
}


//=======================================================================================
//LagPoint
//		point displacement callback
//=======================================================================================
static FX_DISPLACESCAN(DisplaceScan, LagPointState, state)
{
	FXvecd		v, delta;//, siz
	FXdouble	t, d;
	FXmatrix	m;

	//compute vector of the original point using the pivot
	FX_V3SUB(v, p->file_pos, state->piv);

	//get the length of the vector,,,,,,,,,,,,,,,
	d = FX_V3LEN(v);

	//get the weight,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	if (p->flags&FX_DISPFLAG_WEIGHT)
	{
		if (p->weight==0.0) return 2;
		d*=p->weight;
	}

	//compute lag time,,,,,,,,,,,,,,,,,,,,,,,,,,,
	t = state->frame - (d*state->lag);

	//get the world transformation matrix,,,,,,,,
	fxMatrixGet(p->objID, FX_MATRIX_WORLD, t, m, 1);

	//transform the vector,,,,,,,,,,,,,,,,,,,,,,,
	_fxMatrixXform(p->new_pos, v, m, 1);
	FX_V3SUB( delta, p->new_pos, state->loc);
	FX_V3SUB1( delta, p->file_pos );
	FX_V3ADD( p->new_pos, p->cur_pos, delta );

	return FX_TRUE;
}


//=======================================================================================
//EffectScan
//		effect target list scan callback
//			- this function gets called for every target affected by this effect object
//=======================================================================================
static FX_EFFECTSCAN(EffectScan, LagPointState, state)
{
	FXvecd		pos, v, pnt;
	FXdouble	t, d;
	FXint		num_points=0, i;
	FXmatrix	m;

	fxChannelsGet(target, FX_PIVOT, state->frame, state->piv, 0);
	fxChannelsGet(target, FX_WORLD_POSITION, state->frame, state->loc, 0);

	//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	//NOTE: The purpose of this function is to demonstrate point scanning.
	//While it's most efficient to use fxDisplaceScan, you can, however,
	//perform the scanning yourself.  This block of code demonstrates both
	//options, but note that fxDisplaceScan allows you to use a weight.
	//You will be able to use multiple weights in the near future.
	//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	if (state->scan)
	{
		fxDisplaceScan(target, state->weight_obj, FX_WEIGHT_ANIMATE, &DisplaceScan, state, 0);
	}
	else
	{

		fxMeshNumPoints(target, &num_points, FX_NOFLAG);
		fxDisplaceBegin(target, FX_NOFLAG);

			for (i=0;i<num_points;i++)
			{
				//get the point,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
				fxMeshPoint(target, i, pnt, FX_NOFLAG);

				//compute vector of the original point using the pivot
				FX_V3SUB(v, pnt, state->piv);

				//get the length of the vector,,,,,,,,,,,,,,,
				d = FX_V3LEN(v);

				//compute lag time,,,,,,,,,,,,,,,,,,,,,,,,,,,
				t = state->frame - (d*state->lag);

				//get the world transformation matrix,,,,,,,,
				fxMatrixGet(target, FX_MATRIX_WORLD, t, m, 1);

				//transform the vector,,,,,,,,,,,,,,,,,,,,,,,
				_fxMatrixXform(pos, v, m, 1);

				//set the point,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
				fxDisplacePoint(target, i, pos[0], pos[1], pos[2]);
			}

		fxDisplaceEnd(target, FX_NOFLAG);
	}

	return FX_TRUE;
}



