/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_interface.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 10/02/04 - LM
			-- added "flags" to fxInterfaceBlockSet

	-----------------------------------------------------------------
	INTERFACE CREATION
	-----------------------------------------------------------------
	Interface creation for messiah is rather straight-forward.  The
	only element that is "unorthodox" is that your module may have
	multiple interfaces.  Depending on the type of module, you may
	need an interface for Animate, Setup, & Render modes.
	
	Luckily, the API has a mechanism that makes this rather easy.  The
	interface creation sequence is as follows:

		1. Create a block
		2. Create controls for that block
		3. Repeat steps 1 & 2 as desired
		4. Assign those blocks to the relevant interfaces

	EX:
		mi->channels = fxCC_Block("Channels", NULL, ACC_NONE, 0);
			mi->channel1 = fxCC_ChannelFloat("Floopy", NULL, ACC_NONE, FX_CHAN_0, 0);
			mi->channel2 = fxCC_ChannelFloat("Wiz", NULL, ACC_NONE, FX_CHAN_1, 0);

		mi->options =  fxCC_Block("Options", NULL, ACC_UPDATE|ACC_SIZE, 0);
			mi->choice = fxCC_Choice("Decide", NULL, ACC_VALUE, FX_CHAN_0, 0, 2, choices, 0);

		fxInterfaceBlockSet(FX_MODE_ANIMATE, NULL, NULL, mi->channels, 0);
		fxInterfaceBlockSet(FX_MODE_ANIMATE, FX_MODE_ANIMATE, FX_MBLOCK_SPLINE, FX_IDNULL, 0);
		fxInterfaceBlockSet(FX_MODE_SETUP, NULL, NULL, mi->channels, 0);
		fxInterfaceBlockSet(FX_MODE_SETUP, NULL, NULL, mi->options, 0);


	In that example, 2 blocks with controls were created.  Then the
	"channels" block was assigned to both Animate & Setup modes, and
	is the first block in both of those modes.  However, the second
	block for Animate is the standard Spline block.  The "options"
	block is assigned as the second block on Setup.

	Note that extension modules (edit, play, customize) only have
	one interface, so you shouldn't call fxInterfaceBlockSet for
	those modules.  Only call fxInterfaceBlockSet for object-type
	modules (effects & shaders).


	-----------------------------------------------------------------
	USER-OWNED CONTROL TYPES
	-----------------------------------------------------------------
	Some controls serve mainly as a direct connection from the user to
	common data stored in your object.  This means that all you have
	to do is create the control, and all the work of getting/setting
	is handled for you.
	
	An example of this is the fxCC_ChannelFloat control.  Once created,
	it will automatically "connect" to the specified channel.  As the
	user interacts with the control, allmotion editing is already handled.
	In essence, you just create the control, position it, and forget
	about it.

	The current user-owned control types:
		
		  fxCC_ChannelFloat
		  fxCC_ShaderChannelFloat
		  fxCC_KeyFrame
		  fxCC_ColorPicker (non-exclusive, see ColorPicker NOTES below)


	-----------------------------------------------------------------
	CONTROL MESSAGES
	-----------------------------------------------------------------
	When you create a control, you are in control of getting the data
	from user input and setting the data when the control needs updating.
	For example, if you create a boolean control, you must retrieve
	the on/off state when the user clicks the control, and set the
	on/off state whenever the interface needs to be updated.  It all
	starts with control creation.

	EX1: Creating a boolean control

		fxCC_Bool("Option", &MyBoolFunc, ACC_VALUE|ACC_UPDATE);

	As demonstrated in EX1, you must supply a callback and the relevant
	messages to which you will respond.  The ACC_VALUE message is what
	will be sent whenever the user changed the value/state of the control
	and ACC_UPDATE is what will be sent when the interface needs to be
	updates (actually, you will receive ACC_VALUE whether you request it
	or not).  In your callback, all you have to do is respond to those
	messages.

	EX2: Processing message entries

	FX_CONCALLBACK(MyBoolFunc)
	{
		-
		-
		-
		switch(entry)
		{
			case ACC_VALUE:
				fxConGetInt(ctl, &my->options);
				break;

			case ACC_UPDATE:
				fxConSetInt(ctl, my->options);
				break;

			default:
				break;
		}

		return CONTINUE_ENTRY;
	}


	-----------------------------------------------------------------
	CONTROL PARAMTER LINKING
	-----------------------------------------------------------------
	If you're creating an effect, shader, or tool module, you can have
	the control ACC_VALUE and ACC_UPDATE processed automatically.  It
	first requires that you set the control parameter name when creaing
	it.

	EX3: Setting a control parameter name

		#define	PARAM_OPTIONS		"MyEffectOption" //your parameter name
		#define	PARAM_SCALER		"MyEffectScaler" //your parameter name
		-
		-
		-
		ctl = fxCC_Bool("Option", NULL, ACC_VALUE|ACC_UPDATE);
		fxConParam(ctl, PARAM_OPTIONS, FX_UPDATE_CONTROLS, 0);
		ctl = fxCC_Float("Scaler", NULL, ACC_VALUE|ACC_UPDATE, 0);
		fxConParam(ctl, PARAM_SCALER, FX_UPDATE_WORLD|FX_UPDATE_CONTROLS, 0);

	fxConParam sets your param to the control, and will perform the
	requested update when it receives and ACC_VALUE.  Note that since
	we're setting a parameter for the control, we no longer need to
	define an FX_CONCALLBACK for that control.  All we need to do next
	it respond to the O_PARAM* messages in our Access func for the
	object module.

	EX4: Getting/Setting Object Params

		case O_PARAM_SET:
			if (fxArgGetInt(ai, PARAM_OPTIONS, 0, &my->options)) return ENTRY_CONTINUE;
			if (fxArgGetDouble(ai, PARAM_SCALER, 0, &my->value)) return ENTRY_CONTINUE;
			break;

		case O_PARAM_GET:
			if (fxArgSetInt(ai, PARAM_OPTIONS, 0, my->options)) return ENTRY_CONTINUE;
			if (fxArgSetDouble(ai, PARAM_SCALER, 0, my->value)) return ENTRY_CONTINUE;
			break;

	So, when the control receives an ACC_VALUE you will receive an
	O_PARAM_SET whre you will get the parameter from the AccessInfo, and
	when the control receives an ACC_UPDATE, you will receive O_PARAM_GET
	where you will Set the parameter in AccessInfo.

	*See messiah_access.h for further info on requesting access.

	There are many benefits to using parameter linking:

		1. Cleaner and smaller code since you don't have to create
			callbacks for every control.

		2. More error-free code since the getting/setting and interface
			updating will be handled in a consistent manner.

		3. Automatic compatibility with the multi-selection system since
			every object of the module's type will receive the get/set
			messages if selected.

		4. Automatic compatibility with the multiple undo system since
			the params are processed by the Undo filter.

	There are other important benefits that will be revealed in later
	versions of messiah.

	Note that if you specify a callback for a parameter-linked control,
	you will recieve the O_PARAM* messages fist, then your control
	callback will receive the ACC_* messages.  This may be useful if
	you need to perform any additional ops for the control.

	
	-----------------------------------------------------------------
	IMPORANT NOTE: Don't use parameter linking for any non-object
	modules.  Other modules (e.g. Customize, Edit, etc.) are not
	compatible.

	Also, be sure to give your parameters unique names.  Otherwise
	there may be conflicts with other modules sending/receive parameters
	with those same names.
	-----------------------------------------------------------------


=============================================================================*/


#ifdef _MESSIAH_API_INTERFACE

#ifndef __MESSIAH_GUI
#define __MESSIAH_GUI


#ifndef __MESSIAH_HIDE_TYPES


	#define FX_CONCALLBACK(func)	FXintf	func(FXcontrol ctl,  FXentity ctl_data, FX_Arg *arg, FXint64 entry)


	#define ACC_VALUE				((FXint64)0)
	#define ACC_NONE				ACC_VALUE
	#define ACC_UPDATE				ENTRY(0)

	#define ACC_DRAW				ENTRY(1)
	#define ACC_DRAW_POST			ENTRY(2)

	#define ACC_TIMER				ENTRY(3)

	#define ACC_MOUSEDOWN			ENTRY(4)
	#define ACC_MOUSEUP				ENTRY(5)
	#define ACC_MOUSEMOVE			ENTRY(6)
	#define ACC_MOUSEDRAG			ENTRY(7)
	#define ACC_MOUSEDBLCLICK		ENTRY(8)
	#define ACC_MOUSEIN				ENTRY(9)
	#define ACC_MOUSEOUT			ENTRY(10)
	#define ACC_MOUSEWHEEL			ENTRY(11)

	#define ACC_CHAR				ENTRY(12)
	#define ACC_KEYDOWN				ENTRY(13)
	#define ACC_KEYUP				ENTRY(14)

	#define ACC_GAINFOCUS			ENTRY(15)
	#define ACC_LOSEFOCUS			ENTRY(16)

	#define ACC_SIZE				ENTRY(17)
	#define ACC_MOVE				ENTRY(18)
	#define ACC_RENAME				ENTRY(19)

	#define ACC_HIDE				ENTRY(20) //hide control
	#define ACC_SHOW				ENTRY(21) //show control
	#define ACC_REMOVE				ENTRY(22) //remove control
	#define ACC_RESTORE				ENTRY(23) //restore control
	#define ACC_DISABLE				ENTRY(24) //ghost control
	#define ACC_ENABLE				ENTRY(25) //unghost control
	#define ACC_ACTIVATE			ENTRY(26) //make control current
	#define ACC_DEACTIVATE			ENTRY(27) //make control not current

	#define ACC_CREATE				ENTRY(28)
	#define ACC_COPY				ENTRY(29)
	#define ACC_DESTROY				ENTRY(30)

	#define ACC_ALL					ENTRY_ALL


	//Interface Modes,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_MODE_ANIMATE			"Animate"
	#define FX_MODE_SETUP			"Setup"
	#define FX_MODE_RENDER			"Render"
	#define FX_MODE_COMPOSE			"Compose"
	#define FX_MODE_COMMAND			"Command"

	//Standard Blocks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_MBLOCK_VALUES		"Values"
	#define FX_MBLOCK_MOTION		"Motion"
	#define FX_MBLOCK_SPLINE		"Spline"
	#define FX_MBLOCK_IK			"IK"
	#define FX_MBLOCK_SHADER		"Shader"
	#define FX_MBLOCK_PROPERTIES	"Properties"
	#define FX_MBLOCK_WEIGHTDRAW	"WeightDraw"


	//Flags for use with fxInterfaceBlockSet,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_BLOCKSET_DEFAULT		(1<<0)

	//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define CONTROL_HEIGHT			17 //standard control height


	//Block NOTES:----------------------------------------------------------------------------

	#define	BLOCK_MINIMIZED					(1<<0)
	#define BLOCK_NOHEADER					(1<<5)
	#define BLOCK_NOOUTLINE					(1<<6)

	//EX:
	//		bID = fxCC_Block("Name", NULL, ACC_NONE, BLOCK_NOHEADER);
	//
	
	//NOTES:
	//		Use fxConSetInt to Open (1) & Close (0) a block.
	//		Use fxConGetInt to determine if a block is open or closed.


	//Area NOTES:---------------------------------------------------------------------------

	#define AREADRAW_FILL				(1<<0)
	#define AREADRAW_OUTLINE			(1<<1)
	#define AREADRAW_CORNERS			(1<<2)
	#define AREADRAW_BORDER				(1<<3)
	#define AREADRAW_SUNKEN				(1<<4)
	#define AREADRAW_RAISED				(1<<5)

	#define AREADRAW_LABELOVERLAY		(1<<10)
	#define AREADRAW_LABELLEFT			(1<<12)
	#define AREADRAW_LABELRIGHT			(1<<13)
	#define AREADRAW_LABELENGRAVE		(1<<16)
	#define AREADRAW_LABELEMBOSS		(1<<17)

	#define AREADRAW_TRANSPARENT		(1<<18)

	#define AREADRAW_ROUND_BL			(1<<19)
	#define AREADRAW_ROUND_TL			(1<<20)
	#define AREADRAW_ROUND_TR			(1<<21)
	#define AREADRAW_ROUND_BR			(1<<22)

	#define AREADRAW_ROUND_LEFT			(AREADRAW_ROUND_BL|AREADRAW_ROUND_TL)
	#define AREADRAW_ROUND_RIGHT		(AREADRAW_ROUND_TR|AREADRAW_ROUND_BR)
	#define AREADRAW_ROUND_TOP			(AREADRAW_ROUND_TL|AREADRAW_ROUND_TR)
	#define AREADRAW_ROUND_BOTTOM		(AREADRAW_ROUND_BL|AREADRAW_ROUND_BR)
	#define AREADRAW_ROUND_CORNERS		(AREADRAW_ROUND_LEFT|AREADRAW_ROUND_RIGHT)

	//String NOTES:---------------------------------------------------------------------------

	#define STRING_READONLY					(1<<3)
	#define STRING_AUTOCOMPLETE_OBJECT		(1<<4) //send these as GGS_PARAM3 to set the control
	#define STRING_AUTOCOMPLETE_EFFECT		(1<<5) //
	#define STRING_AUTOCOMPLETE_GLOBAL		(1<<6) //
	#define STRING_AUTOCOMPLETE_FUNCTION	(1<<7) //
	#define STRING_AUTOCOMPLETE_SHADER		(1<<8) //
	#define STRING_AUTOCOMPLETE_ALL			(STRING_AUTOCOMPLETE_OBJECT|STRING_AUTOCOMPLETE_EFFECT|STRING_AUTOCOMPLETE_GLOBAL|STRING_AUTOCOMPLETE_FUNCTION|STRING_AUTOCOMPLETE_SHADER)





	//EX: Creating a "read-only" string control
	//
	//		cID = fxCC_String("Name", &Callback, ACC_UPDATE, 100, STRING_READONLY);


	//Choice NOTES:---------------------------------------------------------------------------

	#define CHOICE_VERTICAL					(1<<2)

	//EX:
	//
	//		cID = fxCC_Choice("Name", &Callback, ACC_UPDATE, 100, 1, labels, CHOICE_VERTICAL);


	//DragFloat/Int NOTES:--------------------------------------------------------------------

	#define	MINISLIDER_VERTICAL				(1<<0)

	//EX1: Controls that use vertical minislider
	//
	//		cID = fxCC_DragFloat("Name", &Callback, ACC_VALUE, 0.0, 1.0, 0.01, MINISLIDER_VERTICAL);
	//		cID = fxCC_DragInt("Name", &Callback, ACC_VALUE, 0, 255, 1, MINISLIDER_VERTICAL);

	//EX2: Non-limited range controls (min==max)
	//
	//		cID = fxCC_DragFloat("Name", &Callback, ACC_VALUE, 0.0, 0.0, 0.1, 0);
	//		cID = fxCC_DragInt("Name", &Callback, ACC_VALUE, 0, 0, 1, 0);


	//Drag Button-----------------------------------------------------------------------------

	#define	DRAG_ICON_NONE					0
	#define DRAG_ICON_ROTATE				1
	#define DRAG_ICON_MOVE					2
	#define DRAG_ICON_ZOOM					3

	//EX:
	//		cID = fxCC_DragButton("Name", &Callback, ACC_VALUE, CONTROL_HEIGHT, DRAG_ICON_ROTATE);



	//ChannelFloat & ShaderChannelFloat NOTES:------------------------------------------------

	//EX1: Creating an effect channel control
	//
	//		cID = fxCC_ChannelFloat("Heading", &Callback, ACC_NONE, FX_HEADING, 0);

	//EX2: Creating a shader channel control
	//
	//		cID = fxCC_ShaderChannelFloat("Fog Min Depth", NULL, ACC_NONE, FX_CHAN_3, IN_FOG_MIN_DEPTH, 0);

	//NOTES:
	//		This is a "user-owned" control type (see USER-OWNED CONTROLS above).  That means
	//		that it's not necessary to perform any fxConGet/Set calls.  However, if you want
	//		to change the value of these controls, you must perform motion editing.
	//
	//EX3: Editing a single motion channel
	//
	//		fxKeyEditBegin(yourmoduleID, obj, 0);
	//			fxKeyCreate(obj, NOW, &key, 0);
	//			fxKeyChannelsSet(obj, key, FX_X_POSITION, &val, 0);
	//		fxKeyEditEnd(yourmoduleID, obj, 0);
	//
	//NOTES:
	//		Once you're done editing all motion, you must update the frame & interface
	//
	//EX4: Updating the interface after all motion editing
	//
	//		fxFrameCalculate(NOW, FX_FCALC_UPDATE); //recompute the frame so that the entire frame is accurate
	//		fxInterfaceUpdate(FX_UPDATE_CONTROLS, 0); //update the controls



	//ObjectPopup NOTES:----------------------------------------------------------------------

	#define OBJPOPUP_NONE					(1<<8)  //Always have "(none)" as the first item
	#define OBJPOPUP_ALL					(1<<9)  //Always have "(all)" as the first item
	#define OBJPOPUP_FILTER_GEOMETRY		(1<<10) //Filter objects that have geometry
	#define OBJPOPUP_FILTER_LIGHTS			(1<<11) //Filter lights
	#define OBJPOPUP_FILTER_CAMERAS			(1<<12) //Filter cameras
	#define OBJPOPUP_FILTER_NULLS			(1<<13) //Filter objects that have no geometry.  This includes Nulls, Sliders, Curves, etc.
	#define OBJPOPUP_FILTER_EFFECTS			(1<<14) //Filter effects
	#define OBJPOPUP_FILTER_TOOLS			(1<<15) //Filter tools
	#define OBJPOPUP_FILTER_SUBTOOLS		(1<<16) //Filter subtools
	#define OBJPOPUP_FILTER_TYPESHOW		(1<<18) //use **filter_type as a show list (instead of a filter)
	#define OBJPOPUP_FILTER_SHADERS			(1<<20) //Filter shaders
	#define OBJPOPUP_GETOBJECT				(1<<31) //Allow user to get current object

	//EX:
	//		cID = fxCC_ObjectPopup("Name", &Callback, ACC_UPDATE, 100, NULL, NULL, NULL, OBJPOPUP_FILTER_NULLS|OBJPOPUP_FILTER_CAMERAS|OBJPOPUP_FILTER_SHADERS);


	//Separator NOTES:------------------------------------------------------------------------

	#define SEPARATOR_EMBOSS				(1<<0)
	#define SEPARATOR_ENGRAVE				(1<<1)
	#define SEPARATOR_STARTTOBLOCK			(1<<2) //start of line locks to the left edge of the block
	#define SEPARATOR_ENDTOBLOCK			(1<<3) //end of the line locks to the right edge of the block

	//ColorPicker NOTES:----------------------------------------------------------------------

	#define CPICKER_VALUES		(1<<0) //show color value minislider controls
	#define CPICKER_RGBA		(1<<1) //RGBA mode, otherwise just RGB
	#define CPICKER_CHAN_R		(1<<2) //connect RED component to a motion channel
	#define CPICKER_CHAN_G		(1<<3) //connect GREEN component to a motion channel
	#define CPICKER_CHAN_B		(1<<4) //connect BLUE component to a motion channel
	#define CPICKER_CHAN_A		(1<<5) //connect ALPHA component to a motion channel
	#define CPICKER_CHAN_RGB	(CPICKER_CHAN_R|CPICKER_CHAN_G|CPICKER_CHAN_B)
	#define CPICKER_CHAN_RGBA	(CPICKER_CHAN_RGB|CPICKER_CHAN_A)
	#define CPICKER_SHOWCOLOR	(1<<6) //show a sample of the current color

	//EX1: Creating an RGBA color picker
	//		
	//		cID = fxCC_ColorPicker("Name", &Callback, ACC_UPDATE, 250, 5, NULL, CPICKER_RGBA|CPICKER_VALUES|CPICKER_SHOWCOLOR);
	//

	//EX2: Creating an RGB color picker linked to motion channels
	//
	//		FXveci4		obj_channels;
	//
	//		obj_channels[0] = FX_CHAN_0;
	//		obj_channels[1] = FX_CHAN_1;
	//		obj_channels[2] = FX_CHAN_2;
	//		obj_channels[3] = 0; //unused since we're not using the Alpha of the ColorPicker, in this example
	//		
	//		cID = fxCC_ColorPicker("Name", &Callback, ACC_UPDATE, 250, 5, channels, CPICKER_CHAN_RGB|CPICKER_VALUES|CPICKER_SHOWCOLOR);
	//

	//NOTES:
	//		Since the ColorPicker control can be used to modify object channels and may or may not
	//		contain alpha data, the ColorPicker requires that you use fxConGetVecd4/fxConSetVecd4
	//		to get and set the current color.
	//		
	//		If you use channel linking for the color picker, it's a good idea to link all relevant channels.
	//		For example, if you're creating an RGB color picker, be sure to link R, G, & B to the desired
	//		channels on your object.  Using a "channel linked" color picker forces the control to be
	//		"user owned", just like the fxCC_Channel & ShaderChannel controls.  This means that it's not
	//		necessary to perform any fxConGetVecd4/fxConSetVecd4 on those controls.


	//WeightCombo NOTES:----------------------------------------------------------------------

	//no flags at this time

	#define fxConGetWeight				fxConGetVeci3
	#define fxConSetWeight				fxConSetVeci3

	#define FX_WCMODIFIED_TOOL(d)		((((d)[2])&1)?1:0) //tool has been changed
	#define FX_WCMODIFIED_TYPE(d)		((((d)[2])&2)?1:0) //type has been changed

	//NOTES:
	//		A WeightCombo control is a combination weight_tool and weight_type modifier.
	//		This provides a single convenient control for choosing a weight and setting
	//		the weight type.
	//		
	//		Because this control is modifying 2 values, it uses the FX_ARG_VECTORI3 type.
	//		To set the value.  The first int is the weight tool's ID, the second is the
	//		weight type:
	//		
	//				FXveci			weight; //you'll need to store/retrieve the weight as FXveci, not FXtool
	//		
	//				-
	//				-
	//				-
	//		
	//				weight[0] is your weight tool ID
	//				weight[1] is FX_WEIGHTTYPE_SETUP or FX_WEIGHTTYPE_ANIMATE or FX_WEIGHTTYPE_COMBO
	//		
	//				-
	//				-
	//				-
	//		
	//				fxConSetVeci3(ctl, weight); //OR your can use fxConSetWeight
	//		
	//		To get the weight:
	//		
	//		
	//				fxConGetVeci3(ctl, weight); //OR your can use fxConGetWeight
	//		
	//		Then, if needed, you can check the third int to determine which value has been modified:
	//		
	//				if (FX_WCMODIFIED_TOOL(weight))... if weight tool has been changed 
	//				if (FX_WCMODIFIED_TYPE(weight))... if weight type has been changed 
	//		
	//		IMPORTANT NOTE!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//			When saving/loading a weight of this format, NEVER use fxArgSave/LoadVeci3!
	//			If you do, the weight tool int will not likely be set.  Instead, use the
	//			provided weight load/save funcs:
	//		
	//				fxArgSaveWeight(ai, name, weight, 0);
	//		
	//				-
	//				-
	//				-
	//		
	//				fxArgLoadWeight(ai, name, weight);
	//		
	//		You can also use the same FX_WCMODIFIED_ checks demonstrated above to verify the loaded params.

#endif //__MESSIAH_HIDE_TYPES



#define FX_UPDATE_GRAPH				(1<<0)
#define FX_UPDATE_WORLD				(1<<1)
#define FX_UPDATE_CONTROLS			(1<<2)
#define FX_UPDATE_LIST				(1<<3)
#define FX_UPDATE_TRANSPORT			(1<<4)
#define FX_UPDATE_FRAME				(1<<5)


FX_API(FXvoid, fxInterfaceUpdate)(FXint update, FXint flags);

FX_API(FXint, fxInterfaceMode)(FXchar *buffer, FXint buffer_length, FXint flags);

FX_API(FXint, fxInterfaceBlockSet)(FXchar *mode_name, FXchar *source, FXchar *block_name, FXcontrol block, FXint flags);


//Control Creation Functions---------------------------------------------------
FX_API(FXcontrol, fxCC_Block)(FXchar *name, FXvoid *callback, FXint64 access, FXint flags);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_Area)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXint height, FXint flags);
//no return, at this time.  Will be added later.

FX_API(FXcontrol, fxCC_Button)(FXchar *name, FXvoid *callback, FXint64 access, FXint width);
//non value returning control type

FX_API(FXcontrol, fxCC_Bool)(FXchar *name, FXvoid *callback, FXint64 access);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_BoolButton)(FXchar *name, FXvoid *callback, FXint64 access, FXint width);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_DragButton)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXint icon);
//no return, at this time.  Will be added later.

FX_API(FXcontrol, fxCC_Choice)(FXchar *name, FXvoid *callback, FXint64 access, FXint size, FXint spacing, FXchar **labels, FXint flags);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_String)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXint flags);
//fxConGet/SetString - FX_ARG_STRING

FX_API(FXcontrol, fxCC_Float)(FXchar *name, FXvoid *callback, FXint64 access, FXint flags);
//fxConGet/SetFloat/Double - FX_ARG_DOUBLE

FX_API(FXcontrol, fxCC_Int)(FXchar *name, FXvoid *callback, FXint64 access, FXint flags);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_DragFloat)(FXchar *name, FXvoid *callback, FXint64 access, FXdouble min, FXdouble max, FXdouble step, FXint flags); //FXfloat, FXdouble
//fxConGet/SetFloat/Double - FX_ARG_DOUBLE

FX_API(FXcontrol, fxCC_DragInt)(FXchar *name, FXvoid *callback, FXint64 access, FXint min, FXint max, FXint step, FXint flags);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_ChannelFloat)(FXchar *name, FXvoid *callback, FXint64 access, FXint channel, FXint flags);
//user-owned - do nothing

FX_API(FXcontrol, fxCC_ShaderChannelFloat)(FXchar *name, FXvoid *callback, FXint64 access, FXint channel, FXint input, FXint flags);
//user-owned - do nothing

FX_API(FXcontrol, fxCC_KeyFrame)(FXchar *name, FXvoid *callback, FXint64 access, FXint flags);
//user-owned - do nothing

FX_API(FXcontrol, fxCC_PopupList)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXchar **items, FXint flags);
//fxConGet/SetInt - FX_ARG_INT

FX_API(FXcontrol, fxCC_WeightPopup)(FXchar *name, FXvoid *callback, FXint64 access, FXint width); //FXobject
//fxConGet/SetObj - FX_ARG_OBJECT

FX_API(FXcontrol, fxCC_WeightCombo)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXint flags);
//fxConGet/SetVeci3 - FX_ARG_VECTORI3 - - see WeightCombo NOTES above.

FX_API(FXcontrol, fxCC_ObjectPopup)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXchar **filter_types, FXobject *objIDs, FXvoid *func, FXint flags); //FXobject
//fxConGet/SetObj - FX_ARG_OBJECT

FX_API(FXcontrol, fxCC_Separator)(FXchar *name, FXvoid *callback, FXint64 access, FXint length, FXint weight, FXint vertical, FXint flags);
//non value returning control type

FX_API(FXcontrol, fxCC_ColorPicker)(FXchar *name, FXvoid *callback, FXint64 access, FXint width, FXint spacing, FXveci4 channelIDs, FXint flags); //FXvecd4
//fxConGet/SetVecd4  FX_ARG_VECTORD4 - - see ColorPicker NOTES above.

//`````````````````````````````````````````````````````````````````````````````


//Control Reset Functions---------------------------------------------------
FX_API(FXint, fxCR_Block)(FXcontrol, FXint reset_args, FXint flags);

FX_API(FXint, fxCR_Area)(FXcontrol, FXint reset_args, FXint width, FXint height, FXint flags);

FX_API(FXint, fxCR_Button)(FXcontrol, FXint reset_args, FXint width);

FX_API(FXint, fxCR_BoolButton)(FXcontrol, FXint reset_args, FXint width);

FX_API(FXint, fxCR_DragButton)(FXcontrol, FXint reset_args, FXint width, FXint icon);

FX_API(FXint, fxCR_Choice)(FXcontrol, FXint reset_args, FXint size, FXint spacing, FXchar **labels, FXint flags);

FX_API(FXint, fxCR_String)(FXcontrol, FXint reset_args, FXint width, FXint flags);

FX_API(FXint, fxCR_Float)(FXcontrol, FXint reset_args, FXint flags);

FX_API(FXint, fxCR_Int)(FXcontrol, FXint reset_args, FXint flags);

FX_API(FXint, fxCR_DragFloat)(FXcontrol, FXint reset_args, FXdouble min, FXdouble max, FXdouble step, FXint flags);

FX_API(FXint, fxCR_DragInt)(FXcontrol, FXint reset_args, FXint min, FXint max, FXint step, FXint flags);

FX_API(FXint, fxCR_ChannelFloat)(FXcontrol, FXint reset_args, FXint channel, FXint flags);

FX_API(FXint, fxCR_ShaderChannelFloat)(FXcontrol, FXint reset_args, FXint channel, FXint input, FXint flags);

FX_API(FXint, fxCR_KeyFrame)(FXcontrol, FXint reset_args, FXint flags);

FX_API(FXint, fxCR_PopupList)(FXcontrol, FXint reset_args, FXint width, FXchar **items, FXint flags);

FX_API(FXint, fxCR_ObjectPopup)(FXcontrol, FXint reset_args, FXint width, FXchar **filter_types, FXint *objIDs, FXvoid *func, FXint flags);

FX_API(FXint, fxCR_WeightPopup)(FXcontrol, FXint reset_args, FXint width);

FX_API(FXint, fxCR_WeightCombo)(FXcontrol, FXint reset_args, FXint width, FXint flags);

FX_API(FXint, fxCR_Separator)(FXcontrol, FXint reset_args, FXint length, FXint weight, FXint vertical, FXint flags);

FX_API(FXint, fxCR_ColorPicker)(FXcontrol, FXint reset_args, FXint width, FXint spacing, FXveci4 channelIDs, FXint flags);
//`````````````````````````````````````````````````````````````````````````````


//Control Parameter Functions--------------------------------------------------
FX_API(FXint, fxConParam)(FXcontrol conID, FXchar *param, FXint update, FXint flags);
//`````````````````````````````````````````````````````````````````````````````


//Control Manipulation Functions-----------------------------------------------
FX_API(FXint, fxConSetSize)(FXcontrol conID, FXint width, FXint height);

FX_API(FXint, fxConSetPos)(FXcontrol conID, FXint x, FXint y);

FX_API(FXint, fxConSetName)(FXcontrol conID, FXchar *name);


FX_API(FXint, fxConGetSize)(FXcontrol conID, FXint *width, FXint *height);

FX_API(FXint, fxConGetPos)(FXcontrol conID, FXint *x, FXint *y);

FX_API(FXint, fxConGetName)(FXcontrol conID, FXchar *name, FXint buffer_length);

FX_API(FXint, fxConGetNameWidth)(FXcontrol conID, FXint *width);
//`````````````````````````````````````````````````````````````````````````````


//Control Management Functions-------------------------------------------------
FX_API(FXint, fxConHide)(FXcontrol conID);

FX_API(FXint, fxConShow)(FXcontrol conID);

FX_API(FXint, fxConDisable)(FXcontrol conID);

FX_API(FXint, fxConEnable)(FXcontrol conID);

FX_API(FXint, fxConUpdate)(FXcontrol conID);
//`````````````````````````````````````````````````````````````````````````````


//Get Control value (ACC_VALUE)------------------------------------------------
FX_API(FXint, fxConGetInt)(FXcontrol conID, FXint *var);

FX_API(FXint, fxConGetDouble)(FXcontrol conID, FXdouble *var);

FX_API(FXint, fxConGetFloat)(FXcontrol conID, FXfloat *var);

FX_API(FXint, fxConGetVeci3)(FXcontrol conID, FXint *var);

FX_API(FXint, fxConGetVecd3)(FXcontrol conID, FXdouble *var);

FX_API(FXint, fxConGetVecf3)(FXcontrol conID, FXfloat *var);

FX_API(FXint, fxConGetVeci4)(FXcontrol conID, FXint *var);

FX_API(FXint, fxConGetVecd4)(FXcontrol conID, FXdouble *var);

FX_API(FXint, fxConGetVecf4)(FXcontrol conID, FXfloat *var);

FX_API(FXint, fxConGetCharRGB)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConGetCharRGBA)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConGetChar)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConGetString)(FXcontrol conID, FXchar *var, FXint buffer_length);

FX_API(FXint, fxConGetObj)(FXcontrol conID, FXobject *var);

FX_API(FXint, fxConGetImage)(FXcontrol conID, FXimage *var);

FX_API(FXint, fxConGetVoid)(FXcontrol conID, FXvoid **var);

FX_API(FXint, fxConGetInt64)(FXcontrol conID, FXint64 *var);
//`````````````````````````````````````````````````````````````````````````````



//Set Control value (ACC_UPDATE)-----------------------------------------------
FX_API(FXint, fxConSetInt)(FXcontrol conID, FXint var);

FX_API(FXint, fxConSetDouble)(FXcontrol conID, FXdouble var);

FX_API(FXint, fxConSetFloat)(FXcontrol conID, FXfloat var);

FX_API(FXint, fxConSetVeci3)(FXcontrol conID, FXint *var);

FX_API(FXint, fxConSetVecd3)(FXcontrol conID, FXdouble *var);

FX_API(FXint, fxConSetVecf3)(FXcontrol conID, FXfloat *var);

FX_API(FXint, fxConSetVeci4)(FXcontrol conID, FXint *var);

FX_API(FXint, fxConSetVecd4)(FXcontrol conID, FXdouble *var);

FX_API(FXint, fxConSetVecf4)(FXcontrol conID, FXfloat *var);

FX_API(FXint, fxConSetCharRGB)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConSetCharRGBA)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConSetChar)(FXcontrol conID, FXchar var);

FX_API(FXint, fxConSetString)(FXcontrol conID, FXchar *var);

FX_API(FXint, fxConSetObj)(FXcontrol conID, FXobject var);

FX_API(FXint, fxConSetImage)(FXcontrol conID, FXimage var);

FX_API(FXint, fxConSetVoid)(FXcontrol conID, FXvoid *var);

FX_API(FXint, fxConSetInt64)(FXcontrol conID, FXint64 var);
//`````````````````````````````````````````````````````````````````````````````



#define _MESSIAH_PTR_INTERFACE	messiah_apicomponent = FX_APIFUNCS_INTERFACE;\
								FX_PTR(fxInterfaceUpdate);\
								FX_PTR(fxInterfaceMode);\
								FX_PTR(fxInterfaceBlockSet);\
								\
								FX_PTR(fxCC_Block);\
								FX_PTR(fxCC_Area);\
								FX_PTR(fxCC_Button);\
								FX_PTR(fxCC_Bool);\
								FX_PTR(fxCC_BoolButton);\
								FX_PTR(fxCC_DragButton);\
								FX_PTR(fxCC_Choice);\
								FX_PTR(fxCC_String);\
								FX_PTR(fxCC_Float);\
								FX_PTR(fxCC_Int);\
								FX_PTR(fxCC_DragFloat);\
								FX_PTR(fxCC_DragInt);\
								FX_PTR(fxCC_ChannelFloat);\
								FX_PTR(fxCC_ShaderChannelFloat);\
								FX_PTR(fxCC_KeyFrame);\
								FX_PTR(fxCC_PopupList);\
								FX_PTR(fxCC_WeightPopup);\
								FX_PTR(fxCC_WeightCombo);\
								FX_PTR(fxCC_ObjectPopup);\
								FX_PTR(fxCC_Separator);\
								FX_PTR(fxCC_ColorPicker);\
								\
								FX_PTR(fxCR_Block);\
								FX_PTR(fxCR_Area);\
								FX_PTR(fxCR_Button);\
								FX_PTR(fxCR_BoolButton);\
								FX_PTR(fxCR_DragButton);\
								FX_PTR(fxCR_Choice);\
								FX_PTR(fxCR_String);\
								FX_PTR(fxCR_Float);\
								FX_PTR(fxCR_Int);\
								FX_PTR(fxCR_DragFloat);\
								FX_PTR(fxCR_DragInt);\
								FX_PTR(fxCR_ChannelFloat);\
								FX_PTR(fxCR_ShaderChannelFloat);\
								FX_PTR(fxCR_KeyFrame);\
								FX_PTR(fxCR_PopupList);\
								FX_PTR(fxCR_WeightPopup);\
								FX_PTR(fxCR_WeightCombo);\
								FX_PTR(fxCR_ObjectPopup);\
								FX_PTR(fxCR_Separator);\
								FX_PTR(fxCR_ColorPicker);\
								\
								FX_PTR(fxConParam);\
								\
								FX_PTR(fxConSetSize);\
								FX_PTR(fxConSetPos);\
								FX_PTR(fxConSetName);\
								FX_PTR(fxConGetSize);\
								FX_PTR(fxConGetPos);\
								FX_PTR(fxConGetName);\
								FX_PTR(fxConGetNameWidth);\
								\
								FX_PTR(fxConHide);\
								FX_PTR(fxConShow);\
								FX_PTR(fxConDisable);\
								FX_PTR(fxConEnable);\
								FX_PTR(fxConUpdate);\
								\
								FX_PTR(fxConGetInt);\
								FX_PTR(fxConGetDouble);\
								FX_PTR(fxConGetFloat);\
								FX_PTR(fxConGetVeci3);\
								FX_PTR(fxConGetVecd3);\
								FX_PTR(fxConGetVecf3);\
								FX_PTR(fxConGetVeci4);\
								FX_PTR(fxConGetVecd4);\
								FX_PTR(fxConGetVecf4);\
								FX_PTR(fxConGetCharRGB);\
								FX_PTR(fxConGetCharRGBA);\
								FX_PTR(fxConGetChar);\
								FX_PTR(fxConGetString);\
								FX_PTR(fxConGetObj);\
								FX_PTR(fxConGetImage);\
								FX_PTR(fxConGetVoid);\
								FX_PTR(fxConGetInt64);\
								\
								FX_PTR(fxConSetInt);\
								FX_PTR(fxConSetDouble);\
								FX_PTR(fxConSetFloat);\
								FX_PTR(fxConSetVeci3);\
								FX_PTR(fxConSetVecd3);\
								FX_PTR(fxConSetVecf3);\
								FX_PTR(fxConSetVeci4);\
								FX_PTR(fxConSetVecd4);\
								FX_PTR(fxConSetVecf4);\
								FX_PTR(fxConSetCharRGB);\
								FX_PTR(fxConSetCharRGBA);\
								FX_PTR(fxConSetChar);\
								FX_PTR(fxConSetString);\
								FX_PTR(fxConSetObj);\
								FX_PTR(fxConSetImage);\
								FX_PTR(fxConSetVoid);\
								FX_PTR(fxConSetInt64)


#endif //__MESSIAH_GUI

#endif //_MESSIAH_API_INTERFACE

